import os
import uuid

# ABOUT FOLDERS & FILES
TMP_DIR = r'/tmp'
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
PROMPT_TEMPLATE_DIR = os.path.join(ROOT_DIR, 'prompt_templates')
TEXTUAL_DB_DIR = os.path.join(ROOT_DIR, 'textual_db')

SYS_PROMPT_TEMPLATE_AS_INTERPRETER_FPATH = os.path.join(PROMPT_TEMPLATE_DIR, 'system_template_as_interpreter.txt')
SYS_PROMPT_TEMPLATE_AS_PARSER_FPATH = os.path.join(PROMPT_TEMPLATE_DIR, 'system_template_as_parser.txt')
USR_PROMPT_TEMPLATE_FOR_INTERPRETER_FPATH = os.path.join(PROMPT_TEMPLATE_DIR, 'user_template_for_interpreter.txt')
USR_PROMPT_TEMPLATE_FOR_PARSER_FPATH = os.path.join(PROMPT_TEMPLATE_DIR, 'user_template_for_parser.txt')
DSSAT_LM_FULL_TEMPLATE_AS_INTERPRETER_FPATH = os.path.join(PROMPT_TEMPLATE_DIR, 'dssatlm_full_template_as_interpreter.txt')

DEFINITIONS_BANK_FPATH = os.path.join(TEXTUAL_DB_DIR, 'bank_of_definitions.txt')
QUESTIONS_BANK_FPATH = os.path.join(TEXTUAL_DB_DIR, 'bank_of_questions.txt')
SAMPLE_DEFN_N_QUESTIONS_COVERED_FPATH = os.path.join(TEXTUAL_DB_DIR, 'sample_dssat_questions.csv')

# ABOUT LLM
DEFAULT_LLM_PARAMS = {
    'temperature': 0.9,
    'max_tokens': 10000,
    'top_p': 0.9,
}


LLM_IDS_CONSIDERED = {
    'llama-3.1-70b': 'llama-3.1-70b-versatile',
    'llama-3.3-70b': 'llama-3.3-70b-versatile',
    'gpt-4o': 'gpt-4o',
}

API_KEYS_REQUIRED = ["GROQ_API_KEY", "OPENAI_API_KEY", "WANDB_API_KEY"]

DEFAULT_WANDB_PROJECT_PARAMS = {
    'project': 'dev-dssatlm-project',
    'job_type': 'dev-dssatlm-QA-pipeline',
    'name': 'run_for_user_' + str(uuid.uuid4())
}


from pydantic import BaseModel, Field, model_validator



class DssatLMParserResponse(BaseModel):
    farm_name: str = Field("-99", description="The name of the farm.")
    crop_name: str = Field("-99", description="The name of the crop.")
    latitude: float = Field(-99, description="The latitude of the farm.")
    longitude: float = Field(-99, description="The longitude of the farm.")
    elevation: float = Field(-99, description="The elevation of the farm.")
    planting_date: str = Field("-99", description="The planting date.")
    is_irrigation_applied: str = Field("-99", description="Indicates if irrigation is applied.")
    irrigation_application: list = Field(default_factory=list, description="List of irrigation applications with date and amount.")
    question_statement: str = Field("-99", description="The question statement.")


class DssatLMInterpreterResponse(BaseModel):
    matched_question_found: str = Field(
        "Not Found", 
        description="The question from the bank of definitions that semantically matches the question statement the most."
    )
    retrieved_answer: str = Field(
        "Not Found", 
        description="The extracted the `key-value` pairs in the JSON object that corresponds to the matched question. In the form of `The <key> is <value>`."
    )
    answer_for_farmer: str = Field(
        "Sorry, your answer cannot be answered at the moment.", 
        description="The answer generated by the model for the farmer in a farmer-friendly language."
    )
    
